/* $Id: kb_ui.c,v 1.9 2006-01-09 19:32:26 chris Exp $
 *
 * keyboard emulation
 * (c) 2002,2003,2005,2006 Christian Groessler (chris@groessler.org)
 */

#include <kos.h>
#include "atari.h"
#include "config.h"
#include "input.h"
#include "ui.h"
#include "kb_ui.h"
#include "ui_basic.h"

#ifdef KB_UI

int inject_key = AKEY_NONE;

static int tab_cur_x = 0;
static int tab_cur_y = -1;
static int tab_old_x = -1;
static int tab_old_y = 0;
static int shift_flag;
static int ctrl_flag;
static int old_xb, old_xt;
static int y_disp;
static int min_y;

#define STARTY 6
#define STARTX 3

#define TAB_MAX_Y 6
#define TAB_MAX_X 14

static struct kb_entry keyb_tab[2][TAB_MAX_Y][TAB_MAX_X] = {
	{  /* shift not active */
		{
			{ AKEY_OPTION, "Option", STARTX + 7,	  STARTY-2, },
			{ AKEY_SELECT, "Select", STARTX + 15,	  STARTY-2, },
			{ AKEY_START, "Start",	 STARTX + 23,	  STARTY-2, },
			{ AKEY_HELP, "Help",	 STARTX + 30,	  STARTY-2, &old_xt, FLAG_HIST },
		},
		{
			{ AKEY_ESCAPE, "Esc",	 STARTX + 1,	  STARTY },
			{ AKEY_1, "1",		 STARTX + 5,	  STARTY },
			{ AKEY_2, "2",		 STARTX + 7,	  STARTY },
			{ AKEY_3, "3",		 STARTX + 9,	  STARTY },
			{ AKEY_4, "4",		 STARTX + 11,	  STARTY },
			{ AKEY_5, "5",		 STARTX + 13,	  STARTY },
			{ AKEY_6, "6",		 STARTX + 15,	  STARTY },
			{ AKEY_7, "7",		 STARTX + 17,	  STARTY },
			{ AKEY_8, "8",		 STARTX + 19,	  STARTY },
			{ AKEY_9, "9",		 STARTX + 21,	  STARTY },
			{ AKEY_0, "0",		 STARTX + 23,	  STARTY },
			{ AKEY_LESS, "<",	 STARTX + 25,	  STARTY },
			{ AKEY_GREATER, ">",	 STARTX + 27,	  STARTY },
			{ AKEY_BACKSPACE, "BS",	 STARTX + 29,	  STARTY }
		},
		{
			{ AKEY_TAB, "Tab",	 STARTX,	  STARTY + 2 },
			{ AKEY_q, "Q",		 STARTX + 4,	  STARTY + 2 },
			{ AKEY_w, "W",		 STARTX + 6,	  STARTY + 2 },
			{ AKEY_e, "E",		 STARTX + 8,	  STARTY + 2 },
			{ AKEY_r, "R",		 STARTX + 10,	  STARTY + 2 },
			{ AKEY_t, "T",		 STARTX + 12,	  STARTY + 2 },
			{ AKEY_y, "Y",		 STARTX + 14,	  STARTY + 2 },
			{ AKEY_u, "U",		 STARTX + 16,	  STARTY + 2 },
			{ AKEY_i, "I",		 STARTX + 18,	  STARTY + 2 },
			{ AKEY_o, "O",		 STARTX + 20,	  STARTY + 2 },
			{ AKEY_p, "P",		 STARTX + 22,	  STARTY + 2 },
			{ AKEY_MINUS, "-",	 STARTX + 24,	  STARTY + 2 },
			{ AKEY_EQUAL, "=",	 STARTX + 26,	  STARTY + 2 },
			{ AKEY_RETURN, "RET",	 STARTX + 29,	  STARTY + 2 }
		},
		{
			{ AKEY_CTRL, "Ctrl",	 STARTX,	  STARTY + 4,
			  &ctrl_flag, FLAG_TOGGLE },
			{ AKEY_a, "A",		 STARTX + 5,	  STARTY + 4  },
			{ AKEY_s, "S",		 STARTX + 7,	  STARTY + 4  },
			{ AKEY_d, "D",		 STARTX + 9,	  STARTY + 4  },
			{ AKEY_f, "F",		 STARTX + 11,	  STARTY + 4  },
			{ AKEY_g, "G",		 STARTX + 13,	  STARTY + 4  },
			{ AKEY_h, "H",		 STARTX + 15,	  STARTY + 4  },
			{ AKEY_j, "J",		 STARTX + 17,	  STARTY + 4  },
			{ AKEY_k, "K",		 STARTX + 19,	  STARTY + 4  },
			{ AKEY_l, "L",		 STARTX + 21,	  STARTY + 4  },
			{ AKEY_SEMICOLON, ";",	 STARTX + 23,	  STARTY + 4  },
			{ AKEY_PLUS, "+",	 STARTX + 25,	  STARTY + 4  },
			{ AKEY_ASTERISK, "*",	 STARTX + 27,	  STARTY + 4  },
			{ AKEY_CAPSTOGGLE, "CAPS", STARTX + 29,	    STARTY + 4	}
		},
		{
			{ AKEY_SHFT, "Shft",	 STARTX + 1,	  STARTY + 6,
			  &shift_flag, FLAG_TOGGLE },
			{ AKEY_z, "Z",		 STARTX + 6,	  STARTY + 6 },
			{ AKEY_x, "X",		 STARTX + 8,	  STARTY + 6 },
			{ AKEY_c, "C",		 STARTX + 10,	  STARTY + 6 },
			{ AKEY_v, "V",		 STARTX + 12,	  STARTY + 6 },
			{ AKEY_b, "B",		 STARTX + 14,	  STARTY + 6 },
			{ AKEY_n, "N",		 STARTX + 16,	  STARTY + 6 },
			{ AKEY_m, "M",		 STARTX + 18,	  STARTY + 6 },
			{ AKEY_COMMA, ",",	 STARTX + 20,	  STARTY + 6 },
			{ AKEY_FULLSTOP, ".",	 STARTX + 22,	  STARTY + 6 },
			{ AKEY_SLASH, "/",	 STARTX + 24,	  STARTY + 6 },
			{ AKEY_SHFT, "Shft",	 STARTX + 27,	  STARTY + 6,
			  &shift_flag, FLAG_TOGGLE },
			{ AKEY_ATARI, "a",	 STARTX + 32,	  STARTY + 6 },
		},
		{
			{ AKEY_SPACE, "<<<<<< Space >>>>>>", STARTX + 7, STARTY + 8, &old_xb, FLAG_HIST },
		}
	},
	{   /* shift active */
		{
			{ AKEY_OPTION, "Option", STARTX + 7,	  STARTY-2 },
			{ AKEY_SELECT, "Select", STARTX + 15,	  STARTY-2 },
			{ AKEY_START, "Start",	 STARTX + 23,	  STARTY-2 },
			{ AKEY_HELP, "Help",	 STARTX + 30,	  STARTY-2 },
		},
		{
			{ AKEY_ESCAPE, "Esc",	  STARTX + 1,	   STARTY },
			{ AKEY_EXCLAMATION, "!",  STARTX + 5,	   STARTY },
			{ AKEY_DBLQUOTE, "\"",	  STARTX + 7,	   STARTY },
			{ AKEY_HASH, "#",	  STARTX + 9,	   STARTY },
			{ AKEY_DOLLAR, "$",	  STARTX + 11,	   STARTY },
			{ AKEY_PERCENT, "%",	  STARTX + 13,	   STARTY },
			{ AKEY_AMPERSAND, "&",	  STARTX + 15,	   STARTY },
			{ AKEY_QUOTE, "'",	  STARTX + 17,	   STARTY },
			{ AKEY_AT, "@",		  STARTX + 19,	   STARTY },
			{ AKEY_PARENLEFT, "(",	  STARTX + 21,	   STARTY },
			{ AKEY_PARENRIGHT, ")",	  STARTX + 23,	   STARTY },
			{ AKEY_CLEAR, "c",	  STARTX + 25,	   STARTY },
			{ AKEY_INSERT_LINE, "i",  STARTX + 27,	   STARTY },
			{ AKEY_DELETE_LINE, "DL", STARTX + 29,	   STARTY }
		},
		{
			{ AKEY_TAB, "Tab",	 STARTX,	  STARTY + 2 },
			{ AKEY_Q, "Q",		 STARTX + 4,	  STARTY + 2 },
			{ AKEY_W, "W",		 STARTX + 6,	  STARTY + 2 },
			{ AKEY_E, "E",		 STARTX + 8,	  STARTY + 2 },
			{ AKEY_R, "R",		 STARTX + 10,	  STARTY + 2 },
			{ AKEY_T, "T",		 STARTX + 12,	  STARTY + 2 },
			{ AKEY_Y, "Y",		 STARTX + 14,	  STARTY + 2 },
			{ AKEY_U, "U",		 STARTX + 16,	  STARTY + 2 },
			{ AKEY_I, "I",		 STARTX + 18,	  STARTY + 2 },
			{ AKEY_O, "O",		 STARTX + 20,	  STARTY + 2 },
			{ AKEY_P, "P",		 STARTX + 22,	  STARTY + 2 },
			{ AKEY_UNDERSCORE, "_",	 STARTX + 24,	  STARTY + 2 },
			{ AKEY_BAR, "|",	 STARTX + 26,	  STARTY + 2 },
			{ AKEY_RETURN, "RET",	 STARTX + 29,	  STARTY + 2 }
		},
		{
			{ AKEY_CTRL, "Ctrl",	   STARTX,	    STARTY + 4,
			  &ctrl_flag, FLAG_TOGGLE },
			{ AKEY_A, "A",		   STARTX + 5,	    STARTY + 4	},
			{ AKEY_S, "S",		   STARTX + 7,	    STARTY + 4	},
			{ AKEY_D, "D",		   STARTX + 9,	    STARTY + 4	},
			{ AKEY_F, "F",		   STARTX + 11,	    STARTY + 4	},
			{ AKEY_G, "G",		   STARTX + 13,	    STARTY + 4	},
			{ AKEY_H, "H",		   STARTX + 15,	    STARTY + 4	},
			{ AKEY_J, "J",		   STARTX + 17,	    STARTY + 4	},
			{ AKEY_K, "K",		   STARTX + 19,	    STARTY + 4	},
			{ AKEY_L, "L",		   STARTX + 21,	    STARTY + 4	},
			{ AKEY_COLON, ":",	   STARTX + 23,	    STARTY + 4	},
			{ AKEY_BACKSLASH, "\\",	   STARTX + 25,	    STARTY + 4	},
			{ AKEY_CIRCUMFLEX, "^",	   STARTX + 27,	    STARTY + 4	},
			{ AKEY_CAPSTOGGLE, "CAPS", STARTX + 29,	    STARTY + 4	}
		},
		{
			{ AKEY_SHFT, "Shft",	 STARTX + 1,	  STARTY + 6,
			  &shift_flag, FLAG_TOGGLE },
			{ AKEY_Z, "Z",		 STARTX + 6,	  STARTY + 6 },
			{ AKEY_X, "X",		 STARTX + 8,	  STARTY + 6 },
			{ AKEY_C, "C",		 STARTX + 10,	  STARTY + 6 },
			{ AKEY_V, "V",		 STARTX + 12,	  STARTY + 6 },
			{ AKEY_B, "B",		 STARTX + 14,	  STARTY + 6 },
			{ AKEY_N, "N",		 STARTX + 16,	  STARTY + 6 },
			{ AKEY_M, "M",		 STARTX + 18,	  STARTY + 6 },
			{ AKEY_BRACKETLEFT, "[",       STARTX + 20,	STARTY + 6 },
			{ AKEY_BRACKETRIGHT, "]",    STARTX + 22,     STARTY + 6 },
			{ AKEY_QUESTION, "?",	    STARTX + 24,     STARTY + 6 },
			{ AKEY_SHFT, "Shft",	 STARTX + 27,	  STARTY + 6,
			  &shift_flag, FLAG_TOGGLE },
			{ AKEY_ATARI, "a",	 STARTX + 32,	  STARTY + 6 },
		},
		{
			{ AKEY_SPACE, "<<<<<< Space >>>>>>", STARTX + 7, STARTY + 8, &old_xb, FLAG_HIST },
		}
	}
};

static void init_kb_screen(void)
{
	int tx, ty;

	for (ty=min_y; ty<TAB_MAX_Y; ty++) {
		for (tx=0; tx<TAB_MAX_X; tx++) {
			if (keyb_tab[shift_flag][ty][tx].x) {  /* entry valid */
				Print(0x9a, 0x94, keyb_tab[shift_flag][ty][tx].name,
				      keyb_tab[shift_flag][ty][tx].x,
				      keyb_tab[shift_flag][ty][tx].y + y_disp, 40);
			}
		}
	}
}

static void show_shifts(void)
{
	if (shift_flag) {
		Print(0x94, 0x9a, " SHIFT ", 3, 16, 40);
	}
	else {
		Print(0x9a, 0x94, "       ", 3, 16, 40);
	}
	if (ctrl_flag) {
		Print(0x94, 0x9a, " CONTROL ", 11, 16, 40);
	}
	else {
		Print(0x9a, 0x94, "         ", 11, 16, 40);
	}
}

static void update_pos(void)
{
	if (tab_old_x != -1) {
		/* reset old highlight */
		Print(0x9a, 0x94, keyb_tab[shift_flag][tab_old_y][tab_old_x].name,
		      keyb_tab[shift_flag][tab_old_y][tab_old_x].x,
		      keyb_tab[shift_flag][tab_old_y][tab_old_x].y + y_disp, 40);
	}
	tab_old_x = tab_cur_x;
	tab_old_y = tab_cur_y;
	Print(0x94, 0x9a, keyb_tab[shift_flag][tab_old_y][tab_old_x].name,
	      keyb_tab[shift_flag][tab_old_y][tab_old_x].x,
	      keyb_tab[shift_flag][tab_old_y][tab_old_x].y + y_disp, 40);
}

static void sanitize(void)
{
	int old_x = tab_cur_x;
	while (! keyb_tab[shift_flag][tab_cur_y][tab_cur_x].x) {
		if (tab_cur_x) {
			tab_cur_x--;
			continue;
		}
		tab_cur_x = 0;
		tab_cur_y = min_y;
	}
	if (keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flags & FLAG_HIST) {
		/* hack for single space key */
		*keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flagdep = old_x;
	}
}

void kb_ui(UBYTE *screen, UBYTE *title, unsigned int flags)
{
	int key;
	int osf;
	int old_ui_is_active = ui_is_active;

#ifdef DEBUG
	printf("kb_ui called\n");
#endif

	ui_is_active = TRUE;

	/*shift_flag = FALSE;*/
	ctrl_flag = FALSE;
	osf = shift_flag;

	if (flags & KB_CONSOL) {
		min_y = 0;
		y_disp = 1;
	}
	else {
		min_y = 1;
		y_disp = 0;
	}
	if (tab_cur_y == -1 || tab_cur_y < min_y) tab_cur_y = min_y;

	BasicUIInit();	 /* get charset */

	ClearScreen();
	if (! title) TitleScreen("Keyboard emulator");
	else TitleScreen(title);

	Box(0x9a, 0x94, 0, 3, 39, 17);
	init_kb_screen();
	show_shifts();

	Box(0x9a, 0x94, 0, 18, 39, 24);

	CenterPrint(0x9a, 0x94, "Dreamcast controller buttons:", 20);
	if (flags & KB_SELECT) {
		CenterPrint(0x9a, 0x94, "A  --  leave with key selected", 21);
		CenterPrint(0x9a, 0x94, "L, R, B  --  leave without selection", 22);
	}
	else {
		CenterPrint(0x9a, 0x94, "A  --  leave with key pressed", 21);
		CenterPrint(0x9a, 0x94, "L, R, B  --  leave without keypress", 22);
	}

	inject_key = AKEY_NONE;

	do {
		update_pos();
		key = GetKeyPress();
#ifdef DEBUG
		printf("kb_ui: got key: %02X\n", key);
#endif
		switch (key) {
		case 0x1c:				/* Up */
			if (keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flags & FLAG_HIST) {
				tab_cur_x = *keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flagdep;
			}
			if (tab_cur_y > min_y) tab_cur_y--;
			sanitize();
			break;
		case 0x1d:				/* Down */
			if (keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flags & FLAG_HIST) {
				tab_cur_x = *keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flagdep;
			}
			if (tab_cur_y < TAB_MAX_Y - 1) tab_cur_y++;
			sanitize();
			break;
		case 0x1e:				/* Left */
			if (tab_cur_x > 0) tab_cur_x--;
			sanitize();
			break;
		case 0x1f:				/* Right */
			if (tab_cur_x < TAB_MAX_X - 1) tab_cur_x++;
			sanitize();
			break;
		case 0x9b:				/* Select */
			if (keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flags & FLAG_TOGGLE) {
				*keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flagdep =
					! *keyb_tab[shift_flag][tab_cur_y][tab_cur_x].flagdep;
				key = 0;  /* don't leave menu */
				if (osf != shift_flag) {
					osf = shift_flag;
					init_kb_screen();
				}
				show_shifts();
			}
			else {
				inject_key = keyb_tab[shift_flag][tab_cur_y][tab_cur_x].akey |
					/*(shift_flag ? AKEY_SHFT : 0) |*/
					(ctrl_flag ? AKEY_CTRL : 0);
			}
			break;
		case 0xa0:				/* DC L/R trigger */
			key = 0x1b;
		case 0x1b:				/* Cancel */
			break;
		default:
			break;
		}
	} while (key != 0x9b && key != 0x1b);

	timer_spin_sleep(250);

	ui_is_active = old_ui_is_active;
}

#endif /* #ifdef KB_UI */
