/* $Id: kb_ui_5200.c,v 1.3 2005-12-27 16:07:50 chris Exp $
 *
 * 5200 controller emulation
 * (c) 2002,2005 Christian Groessler (chris@groessler.org)
 */

#include <kos.h>
#include "atari.h"
#include "config.h"
#include "input.h"
#include "ui.h"
#include "ui_basic.h"
#include "kb_ui.h"

#ifdef KB_UI

static int tab_cur_x = 0;
static int tab_cur_y = 0;
static int tab_old_x = -1;
static int tab_old_y = 0;

#define STARTY 5
#define STARTX 5

#define TAB_MAX_Y 5
#define TAB_MAX_X 3

static struct kb_entry keyb_tab_5200[TAB_MAX_Y][TAB_MAX_X] = {
	{
		{ AKEY_5200_START, "Start",   STARTX + 5,      STARTY },
		{ AKEY_5200_PAUSE, "Pause",   STARTX + 12,     STARTY },
		{ AKEY_5200_RESET, "Reset",   STARTX + 19,     STARTY },
	},
	{
		{ AKEY_5200_1, "1",           STARTX + 7,      STARTY + 2 },
		{ AKEY_5200_2, "2",           STARTX + 14,     STARTY + 2 },
		{ AKEY_5200_3, "3",           STARTX + 21,     STARTY + 2 },
	},
	{
		{ AKEY_5200_4, "4",           STARTX + 7,      STARTY + 4 },
		{ AKEY_5200_5, "5",           STARTX + 14,     STARTY + 4 },
		{ AKEY_5200_6, "6",           STARTX + 21,     STARTY + 4 },
	},
	{
		{ AKEY_5200_7, "7",           STARTX + 7,      STARTY + 6 },
		{ AKEY_5200_8, "8",           STARTX + 14,     STARTY + 6 },
		{ AKEY_5200_9, "9",           STARTX + 21,     STARTY + 6 }, 
	},
	{
		{ AKEY_5200_ASTERISK, "*",    STARTX + 7,      STARTY + 8 },
		{ AKEY_5200_0, "0",           STARTX + 14,     STARTY + 8 },
		{ AKEY_5200_HASH, "#",        STARTX + 21,     STARTY + 8 },
	}
};

static void init_5200_screen(void)
{
	int tx, ty;

	for (ty=0; ty<TAB_MAX_Y; ty++) {
		for (tx=0; tx<TAB_MAX_X; tx++) {
			if (keyb_tab_5200[ty][tx].x) {  /* entry valid */
				Print(0x9a, 0x94, keyb_tab_5200[ty][tx].name,
				      keyb_tab_5200[ty][tx].x, keyb_tab_5200[ty][tx].y,
                                      40);
			}
		}
	}
}

static void update_pos(void)
{
	if (tab_old_x != -1) {
		/* reset old highlight */
		Print(0x9a, 0x94, keyb_tab_5200[tab_old_y][tab_old_x].name,
		      keyb_tab_5200[tab_old_y][tab_old_x].x,
		      keyb_tab_5200[tab_old_y][tab_old_x].y,
                      40);
	}
	tab_old_x = tab_cur_x;
	tab_old_y = tab_cur_y;
	Print(0x94, 0x9a, keyb_tab_5200[tab_old_y][tab_old_x].name,
	      keyb_tab_5200[tab_old_y][tab_old_x].x,
	      keyb_tab_5200[tab_old_y][tab_old_x].y,
              40);
}

#define sanitize()   /* currently not needed */

void kb_ui_5200(UBYTE *screen)
{
	int key;

	ui_is_active = TRUE;
	BasicUIInit();	 /* get charset */

	ClearScreen();
	TitleScreen("5200 Controller emulator");

	Box(0x9a, 0x94, 0, 3, 39, 15);
	init_5200_screen();
	Box(0x9a, 0x94, 0, 16, 39, 24);
	CenterPrint(0x9a, 0x94, "Dreamcast controller buttons:", 18);
	CenterPrint(0x9a, 0x94, "A  --  leave with key pressed", 20);
	CenterPrint(0x9a, 0x94, "L, R, B  --  leave without keypress", 21);

	inject_key = AKEY_NONE;

	do {
		update_pos();
		key = GetKeyPress();
#ifdef DEBUG
		printf("got key: %02X\n", key);
#endif
		switch (key) {
		case 0x1c:				/* Up */
			if (tab_cur_y > 0) tab_cur_y--;
			sanitize();
			break;
		case 0x1d:				/* Down */
			if (tab_cur_y < TAB_MAX_Y - 1) tab_cur_y++;
			sanitize();
			break;
		case 0x1e:				/* Left */
			if (tab_cur_x > 0) tab_cur_x--;
			sanitize();
			break;
		case 0x1f:				/* Right */
			if (tab_cur_x < TAB_MAX_X - 1) tab_cur_x++;
			sanitize();
			break;
		case 0x9b:				/* Select */
			inject_key = keyb_tab_5200[tab_cur_y][tab_cur_x].akey;
			break;
		case 0xa0:				/* DC L/R trigger */
			key = 0x1b;
		case 0x1b:				/* Cancel */
			break;
		default:
			break;
		}
	} while (key != 0x9b && key != 0x1b);

	timer_spin_sleep(250);
	ui_is_active = FALSE;
}

#endif /* #ifdef KB_UI */
